@@ -1,7 +1,7 @@
 # ABSTRACT: take structured address data and format it according to the various global/country rules
 
 package Geo::Address::Formatter;
-$Geo::Address::Formatter::VERSION = '1.58';
+$Geo::Address::Formatter::VERSION = '1.59';
 use strict;
 use warnings;
 use feature qw(say);
@@ -16,6 +16,7 @@ use Try::Tiny;
 use YAML qw(Load LoadFile);
 use utf8;
 
+$Data::Dumper::Sortkeys = 1;
 my $THC = Text::Hogan::Compiler->new;
 my %THT_cache; # a place to store Text::Hogan::Template objects
 
@@ -84,9 +85,27 @@ sub _read_configuration {
             $self->{state_codes} = $rh_c;
         }
         catch {
-            warn "error parsing component configuration: $_";
+            warn "error parsing state codes configuration: $_";
         };
     }
+
+    # get the abbreviations
+    #
+    # for now hard coded just for US
+    # TODO: map the languages to countries
+    #
+    my $abbrv_file = $path . '/abbreviations/en.yaml';
+    $self->{abbreviations} = {};   
+    if ( -e $abbrv_file){
+        try {
+            my $rh_c = LoadFile($abbrv_file);
+            $self->{abbreviations}->{'US'} = $rh_c;
+        }
+        catch {
+            warn "error parsing abbreviation configuration: $_";
+        };
+    }
+
     return;
 }
 
@@ -96,6 +115,8 @@ sub format_address {
     my $rh_components = clone(shift) || return;
     my $rh_options = shift || {};
 
+    # deal with the options
+    # country
     my $cc = $rh_options->{country} 
             || $self->_determine_country_code($rh_components) 
             || '';
@@ -103,7 +124,9 @@ sub format_address {
     if ($cc){
         $rh_components->{country_code} = $cc;
     } 
-    #warn Dumper $rh_components;
+
+    # abbreviate
+    my $abbrv = $rh_options->{abbreviate} // 0;
 
     # set the aliases, unless this would overwrite something
     foreach my $alias (sort keys %{$self->{component_aliases}}){
@@ -144,9 +167,13 @@ sub format_address {
     # add the attention, but only if needed
     my $ra_unknown = $self->_find_unknown_components($rh_components);
     if (scalar(@$ra_unknown)){
-        $rh_components->{attention} = join(', ', map { $rh_components->{$_} } @$ra_unknown);
+        $rh_components->{attention} = 
+            join(', ', map { $rh_components->{$_} } @$ra_unknown);
+    }
+
+    if ($abbrv){
+        $rh_components = $self->_abbreviate($rh_components);
     }
-    #warn Dumper $rh_components;
 
     # get a compiled template
     if (!defined($THT_cache{$template_text})){
@@ -159,9 +186,12 @@ sub format_address {
     $text = $self->_render_template($compiled_template, $rh_components);
     $text = $self->_postformat($text,$rh_config->{postformat_replace});
     $text = $self->_clean($text);
+
+    # all done
     return $text;
 }
 
+# remove duplicates ("Berlin, Berlin"), do replacements and similar
 sub _postformat {
     my $self      = shift;
     my $text      = shift;
@@ -403,8 +433,37 @@ sub _apply_replacements {
     return $rh_components;
 }
 
-# " abc,,def , ghi " => 'abc, def, ghi'
+sub _abbreviate {
+    my $self = shift;
+    my $rh_comp = shift // return;
+    #say Dumper $self->{abbreviations};
+    #say Dumper $rh_comp;
 
+    # do we have now the country?
+    if (!defined($rh_comp->{country_code})){
+        warn "unable to determine country, thus unable to abbreviate";
+        return;
+    }
+
+    # do we have abbreviations for this country?    
+    my $cc = $rh_comp->{country_code};
+    if (!defined($self->{abbreviations}->{$cc})){ 
+        warn "no abbreviations defined for $cc";
+        return;
+    } 
+
+    my $rh_abbr = $self->{abbreviations}->{$cc};
+    foreach my $comp_name (keys %$rh_abbr){
+        next if (!defined($rh_comp->{$comp_name}));
+        foreach my $long (keys %{$rh_abbr->{$comp_name}}){
+            my $short = $rh_abbr->{$comp_name}->{$long};
+            $rh_comp->{$comp_name} =~ s/\b$long\b/$short/;
+        }
+    }
+    return $rh_comp;
+}
+
+# " abc,,def , ghi " => 'abc, def, ghi'
 sub _clean {
     my $self = shift;
     my $out  = shift // return;
@@ -512,7 +571,7 @@ Geo::Address::Formatter - take structured address data and format it according t
 
 =head1 VERSION
 
-version 1.58
+version 1.59
 
 =head1 SYNOPSIS
 
@@ -523,6 +582,30 @@ version 1.58
   my $GAF = Geo::Address::Formatter->new( conf_path => '/path/to/templates' );
   my $components = { ... }
   my $text = $GAF->format_address($components, { country => 'FR' } );
+  # 
+  my $short_text = $GAF->format_address($components, { country => 'FR', abbreviate => 1, });
+
+=head2 new
+
+  my $GAF = Geo::Address::Formatter->new( conf_path => '/path/to/templates' );
+
+Returns one instance. The conf_path is required.
+
+=head2 format_address
+
+  my $text = $GAF->format_address(\%components, \%options );
+
+Given a structures address (hashref) and options (hashref) returns a
+formatted address.
+
+Possible options you are: 
+
+    'country', which should be an uppercase ISO 3166-1:alpha-2 code
+    e.g. 'GB' for Great Britain, 'DE' for Germany, etc.
+    If ommited we try to find the country in the address components.
+
+    'abbreviate', if supplied common abbreviations are applied
+    to the resulting output.
 
 =head1 DESCRIPTION
 
@@ -545,8 +628,8 @@ you need:
   Germany: Avenue Road 12, 45678 Deville
   Latvia: Avenue Road 12, Deville, 45678
 
-It gets more complicated with 100 countries and dozens more address
-components to consider.
+It gets more complicated with 200+ countries and territories and dozens more 
+address components to consider.
 
 This module comes with a minimal configuration to run tests. Instead of
 developing your own configuration please use (and contribute to)
@@ -555,32 +638,13 @@ which includes test cases.
 
 Together we can address the world!
 
-=head1 METHODS
-
-=head2 new
-
-  my $GAF = Geo::Address::Formatter->new( conf_path => '/path/to/templates' );
-
-Returns one instance. The conf_path is required.
-
-=head2 format_address
-
-  my $text = $GAF->format_address(\%components, \%options );
-
-Given a structures address (hashref) and options (hashref) returns a
-formatted address.
-
-The only option you can set currently is 'country' which should
-be an uppercase ISO 3166-1 alpha-2 code, e.g. 'GB' for Great Britain.
-If ommited we try to find the country in the address components.
-
 =head1 AUTHOR
 
 edf <edf@opencagedata.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by Opencage Data Limited.
+This software is copyright (c) 2017 by Opencage Data Limited.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
